#
# Patch for CMFQuickInstaller version 1.5.5, 1.5.6, 1.5.7, 1.5.8, 1.5.9
#


from AccessControl import ClassSecurityInfo
from Products.CMFQuickInstallerTool.QuickInstallerTool import QuickInstallerTool
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.CMFQuickInstallerTool.QuickInstallerTool import not_installed

class PatchedQuickInstallerTool:
    #
    #   Quick Installer
    #

    security = ClassSecurityInfo()

    security.declareProtected(ManagePortal, 'checkProductIsRestricted')
    def checkProductIsRestricted(self, p):
        """ Check whether product 'p' is allowed for installation.
        Restricted product list is found in property cmf_quickinstaller_restrictions.
        This property should be placed in the Zope root to be modified only
        by the Zope instance manager but not by the CMF/Plone instance managers
        """
        # product is not restricted by default
        restricted = 0
        restrictions = getattr(self.getPhysicalRoot(), 'cmf_quickinstaller_restrictions', ())
        if restrictions:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            site_id   = getattr(portal, 'id', '')
            site_path = portal.absolute_url_path()
            if site_id:
                for restriction in restrictions:
                    rproduct, rsites = restriction.split('|')
                    rsites = rsites.split(',')
                    if rproduct == p:
                        site_ids   = [sp for sp in rsites if '/' not in sp]
                        site_paths = [sp for sp in rsites if '/' in sp]
                        restricted = (site_id not in site_ids) and (site_path not in site_paths)
                        if restricted:
                            break
        return restricted

    security.declareProtected(ManagePortal, 'listInstallableProducts')
    def listInstallableProducts(self,skipInstalled=1):
        """List candidate CMF products for
        installation -> list of dicts with keys:(id,hasError,status)
        """
        # reset the list of broken products
        self.errors = {}
        pids = self.Control_Panel.Products.objectIds() + not_installed(self)
        pids = [pid for pid in pids if self.isProductInstallable(pid)]

        if skipInstalled:
            installed=[p['id'] for p in self.listInstalledProducts(showHidden=1)]
            pids=[r for r in pids if r not in installed]

        # the following one line is CMFQIProductsRestriction patch
        pids=[r for r in pids if not self.checkProductIsRestricted(r)]
        
        res=[]
        for r in pids:
            p=self._getOb(r,None)
            if p:
                res.append({'id':r, 'status':p.getStatus(),
                            'hasError':p.hasError()})
            else:
                res.append({'id':r, 'status':'new', 'hasError':0})
        res.sort(lambda x,y: cmp(x.get('id',None),y.get('id',None)))
        return res
